"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createMatchingWordsFilter = exports.IterableHunspellReader = void 0;
const affReader_1 = require("./affReader");
const gensequence_1 = require("gensequence");
const fs = require("fs-extra");
const iconv_lite_1 = require("iconv-lite");
const util_1 = require("./util");
const defaultEncoding = 'UTF-8';
class IterableHunspellReader {
    constructor(src) {
        this.src = src;
        this.aff = src.aff;
    }
    get dic() {
        return this.src.dic;
    }
    set maxDepth(value) {
        this.aff.maxSuffixDepth = value;
    }
    get maxDepth() {
        return this.aff.maxSuffixDepth;
    }
    /** the number of .dic entries */
    get size() {
        return this.src.dic.length;
    }
    /**
     * @internal
     */
    dicWordsSeq() {
        return (0, gensequence_1.genSequence)(this.src.dic).map((line) => {
            const [word, rules] = line.split('/', 2);
            return { word, rules, prefixes: [], suffixes: [] };
        });
    }
    /**
     * iterates through the root words of the dictionary
     */
    iterateRootWords() {
        return this.seqRootWords();
    }
    iterateWords() {
        return this.seqWords();
    }
    /**
     * Iterator for all the words in the dictionary. The words are in the order found in the .dic after the
     * transformations have been applied. Forbidden and CompoundOnly words are filtered out.
     */
    [Symbol.iterator]() {
        return this.wholeWords();
    }
    /**
     * create an iterable sequence of the words in the dictionary.
     *
     * @param tapPreApplyRules -- optional function to be called before rules are applied to a word.
     *                            It is mostly used for monitoring progress in combination with `size`.
     */
    seqAffWords(tapPreApplyRules, maxDepth) {
        return this.seqTransformDictionaryEntries(tapPreApplyRules, maxDepth).concatMap((a) => a);
    }
    /**
     * create an iterable sequence of the words in the dictionary.
     *
     * @param tapPreApplyRules -- optional function to be called before rules are applied to a word.
     *                            It is mostly used for monitoring progress in combination with `size`.
     */
    seqTransformDictionaryEntries(tapPreApplyRules, maxDepth) {
        const seq = (0, gensequence_1.genSequence)(this.src.dic);
        let count = 0;
        const dicWords = tapPreApplyRules ? seq.map((a) => (tapPreApplyRules(a, count++), a)) : seq;
        return dicWords.map((dicWord) => this.aff.applyRulesToDicEntry(dicWord, maxDepth));
    }
    /**
     * Iterator for all the words in the dictionary. The words are in the order found in the .dic after the
     * transformations have been applied. Forbidden and CompoundOnly ARE INCLUDED.
     *
     * @internal
     */
    seqWords() {
        return this.seqAffWords()
            .map((w) => w.word)
            .filter(createMatchingWordsFilter());
    }
    /**
     * Returns an iterable that will only return stand alone words.
     */
    wholeWords() {
        return (this.seqAffWords()
            // Filter out words that are forbidden or only allowed in Compounds.
            .filter((w) => !w.flags.isForbiddenWord && !w.flags.isOnlyAllowedInCompound)
            .map((w) => w.word)
            .filter(createMatchingWordsFilter()));
    }
    /**
     * @internal
     */
    seqRootWords() {
        return this.dicWordsSeq().map((w) => w.word);
    }
    static createFromFiles(affFile, dicFile) {
        return __awaiter(this, void 0, void 0, function* () {
            const aff = yield (0, affReader_1.parseAffFileToAff)(affFile, defaultEncoding);
            const buffer = yield fs.readFile(dicFile);
            const dicFileContent = (0, iconv_lite_1.decode)(buffer, aff.affInfo.SET);
            const dic = dicFileContent
                .split('\n')
                .slice(1) // The first entry is the count of entries.
                .map((a) => a.trim())
                .filter((line) => !!line);
            return new IterableHunspellReader({ aff, dic });
        });
    }
}
exports.IterableHunspellReader = IterableHunspellReader;
function createMatchingWordsFilter() {
    return (0, util_1.filterOrderedList)((a, b) => a !== b);
}
exports.createMatchingWordsFilter = createMatchingWordsFilter;
//# sourceMappingURL=IterableHunspellReader.js.map